using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms; // remove later
using System.IO;
using System.Security.Cryptography;

namespace FF3LE
{
    public class Model
    {
        private byte[] data; public byte[] Data { get { return this.data; } set { this.data = value; } }

        private Program program; public Program Program { get { return this.program; } }
        private ReaedRomSetting offsets; public ReaedRomSetting Offsets { get { return this.offsets; } }

        private bool hasHeader; public bool HasHeader { get { return this.hasHeader; } set { this.hasHeader = value; } }
        private byte[] header;

        private long numBytes = 0;
        private string fileName;
        private LevelModel levelModel; public LevelModel LevelModel { get { return levelModel; } }
        private LCDecomp lcDecomp;
        private byte[] dataHash;
        private long checkSum = 0;

        private byte[][] graphicSets = new byte[82][]; public byte[][] GraphicSets { get { return graphicSets; } set { graphicSets = value; } }
        private byte[] animatedGraphics = new byte[0x8000]; public byte[] AnimatedGraphics { get { return animatedGraphics; } set { animatedGraphics = value; } }
        private byte[][] graphicSetsL3 = new byte[19][]; public byte[][] GraphicSetsL3 { get { return graphicSetsL3; } set { graphicSetsL3 = value; } }
        private byte[][] tileSets = new byte[75][]; public byte[][] TileSets { get { return tileSets; } set { tileSets = value; } }
        private byte[][] tileMaps = new byte[351][]; public byte[][] TileMaps { get { return tileMaps; } set { tileMaps = value; } }
        private byte[][] physicalMaps = new byte[43][]; public byte[][] PhysicalMaps { get { return physicalMaps; } set { physicalMaps = value; } }

        private byte[] wobGraphicSet; public byte[] WobGraphicSet { get { return wobGraphicSet; } set { wobGraphicSet = value; } }
        private byte[] wobTileMap; public byte[] WobTileMap { get { return wobTileMap; } set { wobTileMap = value; } }
        private byte[] wobPhysicalMap; public byte[] WobPhysicalMap { get { return wobPhysicalMap; } set { wobPhysicalMap = value; } }
        private byte[] worGraphicSet; public byte[] WorGraphicSet { get { return worGraphicSet; } set { worGraphicSet = value; } }
        private byte[] worTileMap; public byte[] WorTileMap { get { return worTileMap; } set { worTileMap = value; } }
        private byte[] worPhysicalMap; public byte[] WorPhysicalMap { get { return worPhysicalMap; } set { worPhysicalMap = value; } }
        private byte[] stGraphicSet; public byte[] StGraphicSet { get { return stGraphicSet; } set { stGraphicSet = value; } }
        private byte[] stTileMap; public byte[] StTileMap { get { return stTileMap; } set { stTileMap = value; } }

        private bool editWobGraphicSet; public bool EditWobGraphicSet { get { return editWobGraphicSet; } set { editWobGraphicSet = value; } }
        private bool editWobTileMap; public bool EditWobTileMap { get { return editWobTileMap; } set { editWobTileMap = value; } }
        private bool editWorGraphicSet; public bool EditWorGraphicSet { get { return editWorGraphicSet; } set { editWorGraphicSet = value; } }
        private bool editWorTileMap; public bool EditWorTileMap { get { return editWorTileMap; } set { editWorTileMap = value; } }
        private bool editStGraphicSet; public bool EditStGraphicSet { get { return editStGraphicSet; } set { editStGraphicSet = value; } }
        private bool editStTileMap; public bool EditStTileMap { get { return editStTileMap; } set { editStTileMap = value; } }
        private bool[] editPhysicalMaps = new bool[43]; public bool[] EditPhysicalMaps { get { return editPhysicalMaps; } set { editPhysicalMaps = value; } }

        private byte[] stPaletteSet; public byte[] StPaletteSet { get { return stPaletteSet; } set { stPaletteSet = value; } }

        private bool[] editTileSets = new bool[75]; public bool[] EditTileSets { get { return editTileSets; } set { editTileSets = value; } }
        private bool[] editTileMaps = new bool[351]; public bool[] EditTileMaps { get { return editTileMaps; } set { editTileMaps = value; } }
        private ushort[] tileMapSizes = new ushort[351]; public ushort[] TileMapSizes { get { return tileMapSizes; } set { tileMapSizes = value; } }

        private bool assembleLevels = false; public bool AssembleLevels { get { return assembleLevels; } set { assembleLevels = value; } }
        private bool assembleFinal = false; public bool AssembleFinal { get { return assembleFinal; } set { assembleFinal = value; } }
        private bool locked = false; public bool Locked { get { return this.locked; } set { this.locked = value; } }

        public Model(Program program)
        {
            this.program = program;
        }

        public void InitOffsets()
        {
            offsets = new ReaedRomSetting(program);
        }

        /************************************************************************************
        * File Handling Methods
        * *********************************************************************************/
        public bool VerifyRom()
        {
            // not implemented yet!
            if (!locked) // and verified
                return true;
            return false;
        }
        public bool HeaderPresent()
        {
            return (numBytes & (long)0x200) == 0x200;
        }
        public bool RemoveHeader()
        {
            try
            {
                hasHeader = true;
                header = new byte[0x200];

                for (int i = 0; i < 0x200; i++)
                    header[i] = data[i];

                byte[] noHeader = new byte[numBytes - 0x200];

                for (int i = 0; i < numBytes - 0x200; i++)
                {
                    noHeader[i] = data[i + 0x200];

                }
                numBytes -= 0x200;
                data = noHeader;

                return true;
            }
            catch
            {
                MessageBox.Show("Error removing header, please remove manually");
                return false;
            }
        }
        public long GetFileSize()
        {
            return numBytes;
        }
        public void SetFileName(string fileName)
        {
            this.fileName = fileName;
        }
        public string GetFileName()
        {
            return fileName;
        }
        public string GetFileNameWithoutPath()
        {
            try
            {
                return fileName.Substring(fileName.LastIndexOf('\x5c') + 1);
            }
            catch
            {
                return null;
            }
        }
        public string GetFileNameWithoutPathOrExtension()
        {
            string ret = fileName.Substring(fileName.LastIndexOf('\x5c') + 1);
            return ret.Substring(0, ret.LastIndexOf('.'));
        }
        public string GetPathWithoutFileName()
        {
            return fileName.Substring(0, fileName.LastIndexOf('\x5c') + 1);
        }
        public string GetEditorPath()
        {
            return Application.ExecutablePath;
        }
        public string GetEditorPathWithoutFileName()
        {
            return GetEditorPath().Substring(0, GetEditorPath().LastIndexOf('\\') + 1);
        }
        public string GetRomName()
        {
            if (HeaderPresent())
                return ByteToStr(ByteManage.GetByteArray(data, 0x101c0, 21));
            return ByteToStr(ByteManage.GetByteArray(data, 0xffc0, 21));
        }
        public string RomChecksum()
        {
            checkSum = 0;
            for (int i = 0; i < data.Length; i++)
                checkSum += data[i];
            checkSum &= 0xFFFF;
            if ((ushort)checkSum == ByteManage.GetShort(data, offsets.CheckSumAddress))//0x06EC73))
                return "0x" + checkSum.ToString("X") + " (OK)";
            else
                return "0x" + checkSum.ToString("X") + " (FAIL)";
        }
        public ushort RomChecksumBin()
        {
            checkSum = 0;
            for (int i = 0; i < data.Length; i++)
                checkSum += data[i];
            checkSum &= 0xFFFF;
            return (ushort)checkSum;
        }

        /*
         * This code does not work because we essentially are 
         * modifying the data after calculating the new checksum thus changing it
         */
        public void CalculateAndSetNewRomChecksum()
        {
            int check = 0;

            for (int i = 0; i < data.Length; i++)
                check += data[i];
            check &= 0xFFFF;

            ByteManage.SetShort(data, 0x00FFDE, (ushort)check);
        }
        public void CreateNewMD5Checksum()
        {
            MD5 md5Hasher = MD5.Create();

            if (data != null)
                dataHash = md5Hasher.ComputeHash(data);
        }
        public bool VerifyMD5Checksum()
        {
            MD5 md5Hasher = MD5.Create();
            byte[] hash;

            if (dataHash != null)
                hash = md5Hasher.ComputeHash(data);
            else
                return true;

            for (int i = 0; i < dataHash.Length && i < hash.Length; i++)
                if (dataHash[i] != hash[i])
                    return false;

            return true;
        }
        public string GameCode()
        {
            //return ByteToStr(ByteManage.GetByteArray(data, 0xFFC0, 4));
            return ByteToStr(ByteManage.GetByteArray(data, offsets.GameCodeAddress, offsets.GameCodeSize));
        }
        public bool ReadRom()
        {
            try
            {
                FileInfo fInfo = new FileInfo(fileName);
                numBytes = fInfo.Length;
                FileStream fStream = new FileStream(fileName, FileMode.Open, FileAccess.Read);
                BinaryReader br = new BinaryReader(fStream);
                data = br.ReadBytes((int)numBytes);
                br.Close();
                fStream.Close();

                return true;

            }
            catch (Exception e)
            {
                MessageBox.Show("FF6LE was unable to load the rom :" + e.Message);

                fileName = "Invalid File";
                return false;
            }

        }

        public void bugFix()
        {
            data[0x22328] = 0x17;
            data[0x2233B] = 0x55;
            data[0x2233D] = 0x48;
            data[0x2233E] = 0x80;
            data[0x2233F] = 0x43;
            data[0x22340] = 0xB9;
            data[0x22341] = 0x54;
            data[0x22342] = 0x3B;
            data[0x22343] = 0x48;
            data[0x22344] = 0xEA;
        }
        public bool WriteRom()
        {
            try
            {
                // if the loaded rom contained a header, use a buffer that starts the data at 0x200
                // and store the header data to the rom
                if (hasHeader && header != null)
                {
                    byte[] temp = new byte[data.Length + 0x200];
                    header.CopyTo(temp, 0);
                    data.CopyTo(temp, 0x200);
                    BinaryWriter binWriter = new BinaryWriter(File.Open(fileName, FileMode.Create));
                    binWriter.Write(temp);
                    binWriter.Close();
                    return true;
                }
                else
                {
                    BinaryWriter binWriter = new BinaryWriter(File.Open(fileName, FileMode.Create));
                    binWriter.Write(data);
                    binWriter.Close();
                    return true;
                }
            }
            catch (Exception e)
            {
                MessageBox.Show("FF6LE was unable to save the rom \n" + e.Message);
                fileName = "Invalid File";
                return false;
            }

        }
        /************************************************************************************
        * Level Editor Methods
        * *********************************************************************************/
        public void CreateLevelModel()
        {
            lcDecomp = new LCDecomp(data);
            levelModel = new LevelModel(data, this);
        }
        public byte[] Decompress(int offset, int maxSize, ref ushort totalSize)
        {
            return lcDecomp.Decompress(offset, maxSize, ref totalSize);
        }
        public byte[] Decompress(int offset, int maxSize)
        {
            return lcDecomp.Decompress(offset, maxSize);
        }
        public int Compress(byte[] source, byte[] dest)
        {
            return lcDecomp.Compress(source, dest);
        }
        public LevelModel GetLevelModel() { return this.levelModel; }
        public void ClearLevelData()
        {
            for (int i = 0; i < graphicSets.Length; i++)
                graphicSets[i] = null;
            for (int i = 0; i < tileSets.Length; i++)
                tileSets[i] = null;
            for (int i = 0; i < tileMaps.Length; i++)
                tileMaps[i] = null;
            for (int i = 0; i < physicalMaps.Length; i++)
                physicalMaps[i] = null;
        }
        /************************************************************************************
        * Private Supporting Methods
        * *********************************************************************************/
        private string ByteToStr(byte[] toStr)
        {
            System.Text.Encoding encoding = System.Text.Encoding.UTF8;

            return encoding.GetString(toStr);
        }

       

    }

}
